/*
 * config.c - fsf
 *
 * last modified: 2000-02-06
 *
 * this source is free
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <oratypes.h>
#include <ociapr.h>
#include <ocidem.h>
#include "fsql.h"

extern ConfigStruct cfg;
extern sword display_mode;
extern sword display_col_names;

sword get_line (FILE *fp, char *line, sword *len)
{
	char c;
	sword ret_val = 0;

	line[0] = '\0';
	*len = 0;

	while (fread (&c, sizeof (char), 1, fp)) {
		ret_val = 1;
		if (c == '\n') {
			line[*len] = '\0';
			break;
		}
		else if ((*len + 1) < MAX_CFG_LINE) {
			line[(*len)++] = c;
		}
		else {
			break;
		}
	}
	line[*len] = '\0';

	return ret_val;
}

sword parse_config_line (char *line, sword len)
{
	char col[MAX_CFG_COLS][MAX_CFG_VALUE];
	sword i;
	sword c = 0;
	sword l = 0;
	sword in_space = 0;

	for (i = 0; i < len; ++i) {
		if (line[i] == '#') {
			break;
		}
		else if ((! isspace (line[i])) && (l < MAX_CFG_VALUE)) {
			in_space = 0;
			col[c][l++] = line[i];
		} 
		else if (! in_space) {
			in_space = 1;
			col[c][l] = '\0';
			l = 0;
			++c;
		}
		else {
		}
	}
	col[c][l] = '\0';
	if (l > 0) {
		++c;
	}

	if (c < 2) {
		return 1;
	}

	if ((! *col[0]) || (! *col[1])) {
		return 1;
	} 

	if (strcasecmp ("username", col[0]) == 0) {
		strcpy (cfg.username, col[1]);
	}
	else if (strcasecmp ("password", col[0]) == 0) {
		strcpy (cfg.password, col[1]);
	}

	if (strcasecmp ("mode_tty", col[0]) == 0) {
		if (strcasecmp (col[1], "table") == 0) {
			cfg.display_mode_tty = MODE_TABLE;
		}
		else if (strcasecmp (col[1], "by_row") == 0) {
			cfg.display_mode_tty = MODE_BY_ROW;
		}
		else if (strcasecmp (col[1], "by_col") == 0) {
			cfg.display_mode_tty = MODE_BY_COL;
		}
	}

	if (strcasecmp ("mode_notty", col[0]) == 0) {
		if (strcasecmp (col[1], "table") == 0) {
			cfg.display_mode_notty = MODE_TABLE;
		}
		else if (strcasecmp (col[1], "by_row") == 0) {
			cfg.display_mode_notty = MODE_BY_ROW;
		}
		else if (strcasecmp (col[1], "by_col") == 0) {
			cfg.display_mode_notty = MODE_BY_COL;
		}
	}

	if (strcasecmp ("col_names_tty", col[0]) == 0) {
		if (col[1][0] == '1') {
			cfg.display_col_names_tty = 1;
		}
		else {
			cfg.display_col_names_tty = 0;
		}
	}

	if (strcasecmp ("col_names_notty", col[0]) == 0) {
		if (col[1][0] == '1') {
			cfg.display_col_names_notty = 1;
		}
		else {
			cfg.display_col_names_notty = 0;
		}
	}

	if (strcasecmp ("cache_results", col[0]) == 0) {
		if (col[1][0] == '1') {
			cfg.cache_results = 1;
		}
		else {
			cfg.cache_results = 0;
		}
	}


	return 0;
}

void open_config (void)
{
	FILE *fp;
	char *HOME;
	char cfg_filename[FILENAME_MAX];
	char line[MAX_CFG_LINE];
	sword len = 0;
	sword code;

	memset (&cfg, 0, sizeof (cfg));

	/* set defaults */
	cfg.display_col_names_tty = 1;
	cfg.display_col_names_notty = 1;
	cfg.display_mode_tty = MODE_TABLE;
	cfg.display_mode_notty = MODE_BY_ROW;
	cfg.cache_results = 1;
	
	HOME = (char *) getenv ("HOME");
	if (! HOME) {
		return;
	}

	sprintf (cfg_filename, "%s/.fsqlrc", HOME);

	fp = fopen (cfg_filename, "r");
	if (! fp) {
		return;
	}

	while ((code = get_line (fp, line, &len)) != 0) {
		if (line && (*line != '\0')) {
			parse_config_line (line, len);
		}
	}

	fclose (fp);
}

void display_env (void)
{
	char mode_tty_string[16];
	char mode_notty_string[16];

	switch (cfg.display_mode_tty) {
	case MODE_TABLE:
		strcpy (mode_tty_string, "table");
		break;
	case MODE_BY_ROW:
		strcpy (mode_tty_string, "by_row");
		break;
	case MODE_BY_COL:
		strcpy (mode_tty_string, "by_col");
		break;
	default:
		strcpy (mode_tty_string, "table");
		break;
	}

	switch (cfg.display_mode_notty) {
	case MODE_TABLE:
		strcpy (mode_notty_string, "table");
		break;
	case MODE_BY_ROW:
		strcpy (mode_notty_string, "by_row");
		break;
	case MODE_BY_COL:
		strcpy (mode_notty_string, "by_col");
		break;
	default:
		strcpy (mode_notty_string, "by_row");
		break;
	}

	printf ("\n");

	printf ("mode_tty	%s\n", mode_tty_string);
	printf ("mode_notty	%s\n", mode_notty_string);

	printf ("col_names_tty	%d\n", cfg.display_col_names_tty);
	printf ("col_names_notty	%d\n", cfg.display_col_names_notty);

	printf ("cache_results	%d\n", cfg.cache_results);

	printf ("\n");
}
